// Usage:
// $ root -l
// ....
// [0] .L PlotPhiDep.C
// [1] PlotPhiDep("sg.dat")->Draw("apc2);
//
//
//
class Data {
public:
  Data(double phi = 0., double sigd = 0., double sigx = 0.)
       : fPhi(phi), fSigd(sigd), fSigx(sigx) 
  {
    //cerr << "phi = " << fPhi << " sigd = " << fSigd << " sigx = " << fSigx << endl;
  }

  inline double GetPhi () const { return fPhi;  }
  inline double GetSigd() const { return fSigd; }
  inline double GetSigx() const { return fSigx; }


  void Print() const
  {
    cerr << setw(6) << setprecision(3) << setiosflags(ios::fixed) << fPhi  << " "
         << setw(5) << setprecision(2) << setiosflags(ios::fixed) << fSigd << " "
         << setw(8) << setprecision(5) << setiosflags(ios::fixed) << fSigx
         << endl;
  }

private:
  double fPhi;  // phi     [rad]
  double fSigd; // sigma_d [mm]
  double fSigx; // sigma_x [mm]
};

bool operator<(const Data &a, const Data &b)
{
  return a.GetPhi() < b.GetPhi();
}

double Fun(const double *xv, const double *pv)
{
  double sg2 = pv[0]*pv[0]
	     + pv[3]*pv[3]*xv[0]/(pv[1]/cos(pv[2]));
  return sqrt(sg2);
}

TMultiGraph *PlotDlenDep(const char *fname = "sg.dat", bool isg = true)
{
  stringstream slscmd;
  slscmd << "sort -k 1 " << fname << ends; 
  TString list = gSystem->GetFromPipe(slscmd.str().data());

  const int kNpt = 100;
  vector<Data> data[kNpt];

  double phi, sigd, sigx;
  double phis = -99999.;
  int    npt   = -1;
  stringstream ins(list.Data());
  // data format:  phi sig_d sig_x 
  while (ins >> phi >> sigd >> sigx) {
    if (phi != phis) { // new phi
      phis = phi; 
      npt++;
    }
    data[npt].push_back(Data(phi, sigd, sigx));
  }
  npt++;

  /////FIXME////////////////////////////
  const double kCd       = 0.096/sqrt(10.); // [mm/sqrt[mm]
  const double kRadToDeg = 180./TMath::Pi();
  const double xfitmin   = 150.;
  const double xfitmax   = 2500.;
  ////FIXME////////////////////////////

  double xmin = 0.;
  double xmax = 600.;
  double ymin = 0.00;
  double ymax = 0.48;

  auto mgp = new TMultiGraph;

  for (int ipt=0; ipt<npt; ipt++) {
    sort(data[ipt].begin(), data[ipt].end()); // sort by phi
    auto ndlen = data[ipt].size();
    auto phi   = data[ipt].at(0).GetPhi();
    double xdata[ndlen], ydata[ndlen];
    int idlen = 0;
    for (const Data &dat : data[ipt]) {
      sigd = dat.GetSigd();
      xdata[idlen] = pow(sigd/kCd,2);
      ydata[idlen] = dat.GetSigx();
#if 0
      dat.Print();
#endif
      idlen++;
    }
    auto gp = new TGraph(idlen,xdata,ydata);
#if 0
    if (!ipt) mgp->Add(gp, "acp");
    else      mgp->Add(gp, "cp");
#else
    mgp->Add(gp);
#endif
  }

  //--
  // Prepare a canvas if requested.
  //--
  TCanvas *c1 = nullptr;
  if (isg) {
     c1 = new TCanvas("c1", "c1",9,31,699,550);
     gStyle->SetOptStat(0);
     c1->ToggleEventStatus();
     c1->SetGridx();
     c1->SetGridy();
     c1->SetBorderSize(2);
     c1->SetLeftMargin(0.1280576);
     c1->SetBottomMargin(0.1343612);

     auto framep = mgp->GetHistogram();
     auto &frame = *framep;
     frame.SetStats(0);
     frame.GetXaxis()->SetLimits(xmin,xmax);
     frame.SetMinimum(ymin);
     frame.SetMaximum(ymax);
     frame.GetXaxis()->SetTitle("drift length [mm]");
     frame.GetXaxis()->SetLabelSize(0.05);
     frame.GetXaxis()->SetTitleSize(0.06);
     frame.GetYaxis()->SetTitle("#sigma_{x} [mm]");
     frame.GetYaxis()->SetLabelSize(0.05);
     frame.GetYaxis()->SetTitleSize(0.06);
     frame.GetYaxis()->SetTitleOffset(1.0);

     mgp->Draw("apc");

     double xtaglo = xmin + 0.7*(xmax-xmin);

     auto glistp = mgp->GetListOfGraphs();
     TIter next(glistp);
     int iphi = 0;
     while (auto gp = static_cast<TGraph *>(next())) {
        auto ytag   = gp->Eval(xtaglo);
        auto ytaglo = ytag - (ymax-ymin)*0.04;

        auto tagp = new TLatex;
        tagp->SetTextFont(132);
        tagp->SetTextAlign(12);
        auto phi = data[iphi].at(0).GetPhi();
        stringstream stext;
#if 0
        stext << "#phi = " << setw(6) << setprecision(3) << phi << ends;
#else
        stext << "#phi = " << setw(3) << setprecision(0) << setiosflags(ios::fixed) << phi*kRadToDeg << "#circ" << ends;
#endif
        tagp->DrawLatex(xtaglo,ytaglo,stext.str().data());
        tagp->Draw();

        auto funp = new TF1("fun",&Fun,xfitmin,xfitmax,4);
        funp->SetParameter(0,0.1);  // [A] term [mm]
        funp->SetParameter(1,30.);  // Neff
        funp->FixParameter(2,phi);  // phi [rad]
        funp->FixParameter(3,kCd);  // Cd [mm/sqrt[mm])
        //gp->Fit(funp,"Q","",-5,5);
        gp->Fit(funp,"","",xfitmin, xfitmax);
#if 1
        cerr << "------------------------------------------ " << endl;
        cerr << setprecision(1) << setiosflags(ios::fixed)
             << "phi = " <<  phi*kRadToDeg << " [deg]"
             << setprecision(2)
             << " Neff(F) = " << funp->GetParameter(1) 
             << " +/- "       << funp->GetParError (1)
             << endl;
        cerr << "------------------------------------------ " << endl;
#endif
        iphi++;
     }
  }
  return mgp;
}



